#!/usr/bin/env python
"""
_RemoveDeuplicates_

Oracle implementation of Workflow.RemoveDuplicates
"""

from WMCore.WMBS.MySQL.Workflow.RemoveDuplicates import RemoveDuplicates as WorkflowRemoveDuplicatesMySQL


class RemoveDuplicates(WorkflowRemoveDuplicatesMySQL):
    pass
