"""
_GetParentInfo_

Oracle implementation of Files.GetParentAndGrandParentInfo
"""




from WMCore.WMBS.MySQL.Files.GetParentInfo import GetParentInfo as GetParentInfoMySQL

class GetParentInfo(GetParentInfoMySQL):
    pass
