"""
_WorkUnit.GetByID_

Oracle implementation of WorkUnit.GetByID
"""

from __future__ import absolute_import, division, print_function

from WMCore.WMBS.MySQL.WorkUnit.GetByID import GetByID as GetByIDMySQL


class GetByID(GetByIDMySQL):
    """
    _WorkUnit.GetByID_

    Oracle implementation of WorkUnit.GetByID
    """

    pass
