@CHARSET "UTF-8";
.mastheadRight {
	float: right;
	text-align: right;
	position: relative;
	padding-right: 1.25em; 
	margin: 0px, 0px, 10px, 0px;
	border: solid green 0px;
	vertical-align: middle;
	width: 300px; 
	font-size: 0.625em; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	font-weight: bold; 
}
.mastheadLeft {
	float: left;
	position: relative;
	padding-left: 0.75em;
	margin: 0px, 0px, 10px, 0px;
	word-spacing: 0.75em;
	border: solid green 0px;	
	vertical-align: middle;
	width: 430px;
 	text-align: left;	
	font-size: 0.625em; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	font-weight: bold; 
}
.masthead {
	width: 100%; 
	top: 0px; 
	left: 0px; 
	position: absolute; 
	border: solid red 1px; 
	padding: 0px; 
	margin: 0px;
	vertical-align: middle;
	line-height: 1.5em;
}
.masthead .mastheadLeft .mastheadLogo {
	border: solid coral 0px;
	height: 16px;
	width: 16px;
	margin-right: 1em;
	vertical-align: middle;
}
.masthead .mastheadLeft ul, .masthead .mastheadRight ul {
	display: inline;
	margin: 0px;
	padding-left: 0.5em;
	vertical-align: middle;
	list-style-type: none;
}
.masthead .mastheadLeft ul li, .masthead .mastheadRight ul li {
	display: inline;
	border: 0px;
	margin: 0px;
	list-style-type: none;
	vertical-align: middle;
}
.masthead .mastheadLeft ul li a, .masthead .mastheadRight ul li a {
	text-decoration: none;
	vertical-align: middle;
	padding: 0.31em 1em 0.31em 1em;
}
.masthead .mastheadLeft ul li a:hover, .masthead .mastheadRight ul li a:hover {
	text-decoration: underline;
}
.masthead .mastheadLeft ul li a:active, .masthead .mastheadRight ul li a:active {
	text-decoration: underline;
}
.masthead #dbs a, .masthead #phedex a, .masthead #prodrequest a, .masthead #dashboard a{
	padding-bottom: 0.75em;	
}
/* This next bit is for site dependant masthead */
.mastheadfooter {
	clear:both;
	left:0px;
	width:100%;
	padding:0em;
	margin:0px;
	border:solid blue 0px;
	line-height: 1.1em;
}
.mastheadfooter ul {
	display: inline;
	border:none blue 0px;
	margin:0px;
	padding-left: 2em;
	font-size: 10px; 
	font-family: 'Lucida Grande', verdana, helvetica, arial, sans-serif;
	vertical-align:middle;
	list-style-type: none;
}
.mastheadfooter ul li {
	display: inline;
	border:none blue 0px;
	margin:0px;
	padding: 0em;
	list-style-type: none;
}
.mastheadfooter ul li a, .mastheadfooter ul li a:visited{
	text-decoration: none;
	border:none blue 0px;
	margin:0px;
	padding: 0em;
}
.mastheadfooter ul li a:hover{
	text-decoration: underline;
	border:none blue 0px;
}
.mastheadfooter ul li a:active{
	text-decoration: underline;
	border:none blue 0px;
}
